<?php

namespace ACL\Services;

use ACL\Helpers\Instances\InstanceProvider;
use ACL\Repositories\Operador as OperadorRepository;
use ACL\Traits\TCrud;
use ACL\Util\ACLSession;
use ACL\Util\AclUtilities;
use ACL\Util\Messages;
use Doctrine\DBAL\ConnectionException;
use Exception;
use Zeedhi\Framework\Remote\FileUploader\Server;
use Teknisa\Libs\Util\Utilities as LibUtilities;

class Operador {

    use TCrud;

    const ACTIVE_ID = 'S';
    const NOT_ACTIVE_ID = 'S';

    /**
     * @var \Teknisa\Libs\Util\Environment
     */
    private $libEnvironment;

    /**
     * @var OperadorRepository
     */
    private $operadorRepository;

    /**
     * @var // FileServerFilesUrl
     */
    private $fileServerFilesUrl;
    /**
     * @var Server
     */
    private $zhFileUploader;

    /**
     * Operador constructor.
     * @param $fileServerFilesUrl
     * @param Server $zhFileUploader
     * @param OperadorRepository $operadorRepository
     * @throws Exception
     */
    public function __construct($fileServerFilesUrl, Server $zhFileUploader,OperadorRepository $operadorRepository)
    {
        $this->fileServerFilesUrl = $fileServerFilesUrl;
        $this->zhFileUploader = $zhFileUploader;
        $this->operadorRepository = $operadorRepository;
        $this->libEnvironment =  InstanceProvider::getLibEnvironment();
    }

    /**
     * @param $params
     * @return array
     * @throws Exception
     */
    public function findAllOperador($params): array
    {
        $cdfiliais = !empty($params['CDFILIAL']) ? $params['CDFILIAL'] : null;
        $filialFilter = $this->buildFilialFilter($cdfiliais);
        $idoperativoFilter = $this->getIdoperativoFilter($params);
        $oAccCustomOrgId = $this->getOrganizationIdFromOAccCustom();
        return $this->operadorRepository->findAllOperador($this->getOrganizationId(), $idoperativoFilter, $oAccCustomOrgId, $filialFilter);
    }

    /**
     * @param $cdfiliais
     * @return string
     */
    private function buildFilialFilter($cdfiliais): string
    {
        if ($cdfiliais) {
            $filiaisString = "'" . implode("','", $cdfiliais) . "'";
            $filiaisString = explode(',', $filiaisString);
            $inQueryString = AclUtilities::fixBugWhereINQuery($filiaisString, 'CDFILIAL');
            return " AND (CDOPERADOR IN (SELECT CDOPERADOR FROM FILIOPER WHERE {$inQueryString}))";
        }
        return '';
    }

    public function createOperador($cdoperador, $nmoperador, $nmacesso, $dsemailoper, $newPassword, $confirmPassword,
                                   $idativo, $idoperativo, $cdpais, $cdcliente, $nrinscricaoparc, $nrparcnegocio,
                                   $cdtipoinscricao, $idoperacesso, $idopermaster, $nrestruturam, $idtpoperador,
                                   $nrorgoper, $cdfiliais, $labels)
    {
        $entityManager = InstanceProvider::getEntityManager();
        $connection = $entityManager->getConnection();
        $parcnegocioService = InstanceProvider::getService()->getParcnegocio();
        $connection->beginTransaction();
        try {

            if (!$cdoperador) {
                $cdoperador = $this->generateCdoperador();
            }
            $this->operadorSaveValidation($cdoperador, $dsemailoper, getNrorg(), $newPassword, $confirmPassword);
            $parcnegocios = $this->operatorSaveValidationCpf($nrinscricaoparc);

            $nrparcnegocio = $this->setParcnegocio($nrparcnegocio, $nmoperador, $nrinscricaoparc, $cdtipoinscricao,
                                                   $idativo, $parcnegocios, $entityManager, $labels);

            $parcnegocioService->createParceria($nrparcnegocio, getNrorg(), getCurrentUserId(), 'OPERADOR', $labels);
            $entityManager->flush();

            if ($cdcliente != null) {
                $parcnegocioService->createParceria($nrparcnegocio, getNrorg(), getCurrentUserId(), 'INTERLOCUTOR', $labels);
                $entityManager->flush();
            }

            $senhaMd5 = md5($newPassword);

            $this->operadorRepository->createOperador(
                $cdoperador, $idoperativo, $idativo, $nmoperador, $nmacesso, $dsemailoper,
                $cdpais, $nrparcnegocio, $cdcliente, getNrorg(), getNrorg(), $idoperacesso,
                getNrorg(), getCurrentUserId(), $senhaMd5, $idopermaster, $nrestruturam,
                $idtpoperador, $nrorgoper);
            $this->saveLogLib($cdoperador, $nmoperador, $cdcliente, $nrparcnegocio, $senhaMd5, $labels);
            $entityManager->flush();

            $this->createOrganizacaooper(getNrorg(), $cdoperador, getCurrentUserId());
            $entityManager->flush();

            $this->operadorRepository->updateParcnegocioInOperador($nrparcnegocio, $cdoperador, getNrorg());
            $entityManager->flush();

            if (!empty($cdfiliais)) {
                $this->createFilioper($cdoperador, $cdfiliais, getNrorg(), getCurrentUserId(), $labels);
            }

            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    public function saveLogLib($operatorCode, $operatorName, $clientCode, $businessPartnership, $password, $labels) {
        $valores = [$labels['label'], $operatorCode, $operatorName, $clientCode, $businessPartnership, '*************'];
        $nomes = ["Tela", $labels['CDOPERADOR'], $labels['NMOPERADOR'], $labels['CDCLIENTE'], $labels['NRPARCNEGOCIO'], $labels['NEW_PASSWORD']];
        $descricao = AclUtilities::gerarDescricao($valores, $nomes);
        InstanceProvider::getLibLog()->logRegister($operatorCode, $descricao, '1', 'OPERADOR', '/operator/save');
    }


    private function createFilioper($cdoperador, $cdfiliais, $nrorg, $cdoperinclusao, $labels)
    {
        foreach ($cdfiliais as $cdfilial) {
            InstanceProvider::getRepository()->getFilioper()->create($cdoperador, $cdfilial, $nrorg, $cdoperinclusao);
            $valores = [$labels['label'], $cdoperador, $cdfilial];
            $nomes = ["Tela", $labels['CDOPERADOR'], $labels['CDFILIAL']];
            $descricao =  AclUtilities::gerarDescricao($valores, $nomes);
            InstanceProvider::getLibLog()->logRegister($cdoperador.'_'.$cdfilial, $descricao, '1', 'FILIOPER', '/operator');
        }
    }

    /**
     * @return array
     * @throws Exception
     */
    public function updateOperador($cdoperador, $nmoperador, $nmacesso, $dsemailoper, $idativo, $idoperativo, $cdpais,
                                   $cdcliente, $nrinscricaoparc, $nrparcnegocio, $cdtipoinscricao, $idopermaster,
                                   $nrestruturam, $idtpoperador, $nrorgoper, $oldCpf, $oldNmoperador, $oldCdcliente, $labels): array
    {
        $entityManager = InstanceProvider::getEntityManager();
        $connection = $entityManager->getConnection();
        $connection->beginTransaction();

        try {
            $this->operadorEditValidation($cdoperador, $dsemailoper);
            $parcnegocios = $this->operatorEditValidationCpf($nrinscricaoparc, $nrparcnegocio);

            $NRPARCNEGOCIO = $this->setParcnegocioByEdit($nrparcnegocio, $nmoperador, getNrorg(), $nrinscricaoparc, $cdtipoinscricao, $idativo,
                                        $parcnegocios, $oldCpf, $oldNmoperador, $entityManager, $labels);

            if($oldCdcliente != $cdcliente){
                $this->setChangeClient($cdcliente, $NRPARCNEGOCIO, $entityManager);
            }
            $this->editLogLib($cdoperador, $nmoperador, $cdcliente, $NRPARCNEGOCIO, $idativo, $nmacesso, $dsemailoper, $cdpais, $nrestruturam, $idtpoperador, $labels);
            $this->operadorRepository->editOperator($cdoperador, $idoperativo, $idativo, $nmoperador, $nmacesso,
                                                    $dsemailoper, $cdpais, $cdcliente, getNrorg(), getCurrentUserId(),
                                                    $NRPARCNEGOCIO, $idopermaster, $nrestruturam, $idtpoperador, $nrorgoper);
            $entityManager->flush();
            $connection->commit();
            if ($idativo === 'N') {
                $tokensOperador = $this->operadorRepository->getTokenOperador($cdoperador);
                if (!empty($tokensOperador)) {
                    foreach ($tokensOperador as $tokenOperador) {
                        if ($tokenOperador['TOKEN'] !== null) {
                            InstanceProvider::getOAuthCustom()->deleteToken($tokenOperador['TOKEN']);
                        }
                    }
                }
            }
            return array("CDOPERADOR" => $cdoperador, "NRPARCNEGOCIO"  => $nrparcnegocio);
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    public function editLogLib($operatorCode, $operatorName, $clientCode, $businessPartnership, $activeId, $accessName, 
    $operatorEmail, $countryCode, $nrestruttrab, $idtpoperador, $labels) {
        $operatorAntigo =  $this->operadorRepository->findValAntigoOperators($operatorCode);
        $arrayValores = [$labels['label']];
        $arrayNomes = ['Tela'];
        if ($operatorAntigo[0]['NMOPERADOR'] !== $operatorName) {
            array_push($arrayValores, $operatorName);
            array_push($arrayNomes, $labels['NMOPERADOR']);
        }
        if ($operatorAntigo[0]['CDCLIENTE'] !== $clientCode) {
            array_push($arrayValores, $clientCode);
            array_push($arrayNomes, $labels['CDCLIENTE']);
        }
        if ($operatorAntigo[0]['NRPARCNEGOCIO'] !== $businessPartnership) {
            array_push($arrayValores, $businessPartnership);
            array_push($arrayNomes, $labels['NRPARCNEGOCIO']);
        }
        if ($operatorAntigo[0]['IDATIVO'] !== $activeId) {
            array_push($arrayValores, $activeId);
            array_push($arrayNomes, $labels['IDATIVO']);
        }
        if ($operatorAntigo[0]['NMACESSO'] !== $accessName) {
            array_push($arrayValores, $accessName);
            array_push($arrayNomes, $labels['NMACESSO']);
        }
        if ($operatorAntigo[0]['DSEMAILOPER'] !== $operatorEmail) {
            array_push($arrayValores, $operatorEmail);
            array_push($arrayNomes, $labels['DSEMAILOPER']);
        }
        if ($operatorAntigo[0]['CDPAIS'] !== $countryCode) {
            array_push($arrayValores, $countryCode);
            array_push($arrayNomes, $labels['CDPAIS']);
        }
        if ($operatorAntigo[0]['NRESTRUTTRAB'] !== $nrestruttrab) {
            array_push($arrayValores, $nrestruttrab);
            array_push($arrayNomes, $labels['NMESTRUTURAM']);
        }
        if ($operatorAntigo[0]['IDTPOPERADOR'] !== $idtpoperador) {
            array_push($arrayValores, $idtpoperador);
            array_push($arrayNomes, $labels['IDTPOPERADOR']);
        }
        $descricao = AclUtilities::gerarDescricao($arrayValores, $arrayNomes);
        InstanceProvider::getLibLog()->logRegister($operatorCode, $descricao, '2', 'OPERADOR', '/operator');
    }

    public function findOperadoresByPerfilAcesso($nrperfilacesso, $params): array
    {
        $idoperativoFilter = $this->getIdoperativoFilter($params);
        return $this->operadorRepository->findOperadoresByPerfilAcesso($nrperfilacesso, $this->getOrganizationId(), $idoperativoFilter);
    }

    /**
     * @param $nrperfilacesso
     * @return array
     * @throws Exception
     */
    public function findOperadoresForAddPerfilAcesso($nrperfilacesso): array
    {
        return $this->operadorRepository->findOperadoresForAddPerfilAcesso($nrperfilacesso, $this->getOrganizationId());
    }

    /**
     * @param $nrorgoper
     * @param false $isPrimaryOrg
     * @param null $showFullOperator
     * @param null $search
     * @return array|array[]|mixed
     * @throws \Doctrine\DBAL\Exception
     */
    public function findOrganizacaoRelacionadaByOperador($nrorgoper, &$isPrimaryOrg = false, $showFullOperator = null, $search = null): array
    {
        $filioperRepository = InstanceProvider::getRepository()->getFilioper();
        $organizacaoRepository = InstanceProvider::getRepository()->getOrganizacao();
        $oacctekmgrRepository = InstanceProvider::getRepository()->getOAccTekMgr();

        $idUtlPerfilfilial = $this->libEnvironment->getIdUtlPerfilFilial();
        $oAccTekMgr = $oacctekmgrRepository->findOneBy(array('NRORG' => getNrorg()), getNrorg());
        $oAccTek = @json_decode(LibUtilities::zeedhiDecrypt($oAccTekMgr['OACCTEK_PHP7'], LibUtilities::LOGIN_CRYPT_STRING), true);
        $oAccCustomOrgId = $this->getOrganizationIdFromOAccCustom();

        $data = [];
        if (!empty($oAccTek['relatedOrgs'])) {
            $operator = $this->operadorRepository->findOneBy(array('CDOPERADOR' => getCurrentUserId()));
            if(!empty($operator['NRORGOPER']) && $operator['NRORGOPER'] != 1) {
                $searchValue = $operator['NRORGOPER'];
            } else {
                $searchValue = !empty($oAccCustomOrgId) ? $oAccCustomOrgId : getNrorg();
            }
            $key = array_search($searchValue, array_column($oAccTek['relatedOrgs'], 'NRORG'));
            if ($key !== false) {
                if(($idUtlPerfilfilial && $idUtlPerfilfilial != 'N') && $showFullOperator) {
                    $filterFilioper = "AND CDOPERADOR = '". $operator['CDOPERADOR'] . "'";
                    $relatedBranches = $filioperRepository->findFilioper(getNrorg(), $filterFilioper);

                    $organizacoes = $this->getOrganizacoesRelacionadas($relatedBranches, $oAccTek['relatedOrgs'], $isPrimaryOrg);
                    $data = $organizacoes;
                } else {
                    if ($oAccTek['relatedOrgs'][$key]['NRORGPRINCIPAL'] === 'S') {
                        $data = [$oAccTek['relatedOrgs'][$key]];
                    } else {
                        $data = $oAccTek['relatedOrgs'];
                    }
                }
            }
        } else {
            if (empty($oAccTek['relatedOrgIds'])) {
                $oAccTek['relatedOrgIds'] = [getNrorg()];
            }
            $filter = AclUtilities::getFilterTypeIn($oAccTek['relatedOrgIds'], 'NRORG');
            $data = $organizacaoRepository->findOrganizacaoRelacionadaByOperador($filter);
            if (empty($data)) {
                $filter = AclUtilities::getFilterTypeIn([getNrorg()], 'NRORG');
                $data = $organizacaoRepository->findOrganizacaoRelacionadaByOperador($filter);
            }
        }

        if ($nrorgoper && $nrorgoper != 1 && array_search($nrorgoper, array_column($data, 'NRORG')) === false) {
            return [[
                'NRORG' => $nrorgoper,
                'NMORGANIZACAOOPER' => $nrorgoper . ' - Organização não encontrada',
                'NMRAZSOCIALORG' => $nrorgoper . ' - Organização não encontrada'
            ]];
        } else if (empty($data)) {
            $filter = AclUtilities::getFilterTypeIn([getNrorg()], 'NRORG');
            $data = $organizacaoRepository->findOrganizacaoRelacionadaByOperador($filter);
        }

        return $this->filterOrganizacaoRelacionadaByOperador($data, $search);
    }

    /**
     * @param $data
     * @param $search
     * @return array
     */
    private function filterOrganizacaoRelacionadaByOperador($data, $search): array
    {
        $key = array_search($search, array_column($data, 'NRORG'));
        if ($key) {
            return [$data[$key]];
        }
        return $data;
    }

    private function getOrganizationIdFromOAccCustom() {
        $oAccCustom = array();
        try {
            $fileDir = '../../../login/backend/config/oAccCustom.tek';
            if (file_exists($fileDir)) {
                $fileContents  = file_get_contents($fileDir);
                $fileDecrypted = LibUtilities::zeedhiDecrypt($fileContents, LibUtilities::LOGIN_CRYPT_STRING);
                $fileDecoded   = @json_decode($fileDecrypted, true);
                if(isset($fileDecoded['organizationId']) && is_numeric($fileDecoded['organizationId'])) {
                    $oAccCustom = $fileDecoded;
                }
            }
        } catch(Exception $e) {}

        return !empty($oAccCustom) ? $oAccCustom['organizationId'] : null;
    }

    /**
     * @param $params
     * @return string
     */
    private function getIdoperativoFilter($params): string
    {
        $idoperativoFilter = '';
        if (isset($params['dynamic_filter'])) {
            $dynamicFilter = json_decode(base64_decode($params['dynamic_filter']), true);
            if (isset($dynamicFilter['IDATIVO']) && $dynamicFilter['IDATIVO'][0]['value'] === 'S') {
                $idoperativoFilter = "AND IDOPERATIVO = 'A'";
            } else {
                $idoperativoFilter = "AND IDOPERATIVO = 'I'";
            }
        }
        return $idoperativoFilter;
    }

    /**
     * @return mixed|null
     * @throws Exception
     */
    private function getOrganizationId()
    {
        return InstanceProvider::getLibEnvironment()->getCurrentOrganizationId();
    }

    /**
     * @param $relatedBranches
     * @param $orgs
     * @param $isPrimaryOrg
     * @return array|mixed
     */
    private function getOrganizacoesRelacionadas($relatedBranches, $orgs, &$isPrimaryOrg): array
    {
        $cnpjEqualArray = [];
        foreach($relatedBranches as $branch){
            $isCnpEqual = array_filter($orgs, function($o) use ($branch, &$isPrimaryOrg){
                if($o['NRINSJUREMPR'] == $branch['NRINSJURFILI']) {

                    if($o['NRORGPRINCIPAL'] == 'P'){
                        $isPrimaryOrg = true;
                    }
                    return $o;
                } else {
                    return false;
                }
            });
            if($isCnpEqual){
                $cnpjEqualArray[] = array_values($isCnpEqual)[0];
            }
        }

        return $isPrimaryOrg ? $orgs : $cnpjEqualArray;
    }

    /**
     * @param $cdoperador
     * @param $email
     * @param $organizationId
     * @param $password
     * @param $confirmPassword
     * @throws Exception
     */
    private function operadorSaveValidation($cdoperador, $email, $organizationId, $password, $confirmPassword)
    {
        $msgError = '';
        $operador = $this->operadorRepository->findOperadorByCdoperador($cdoperador, getNrorg());

        if(!empty($operador)) {
            if($operador[0]['NRORG'] == $organizationId) {
                $msgError .= Messages::ERR_EXISTS_OPERATOR_CODE;
            } else {
                $msgError .= Messages::ERR_EXISTS_OPERATOR_CODE_DIFFERENT_ORGANIZATION;
            }
        }

        $organizationId = getNrorg();
        if($this->findCount("OPERADOR", "WHERE DSEMAILOPER = '{$email}' AND NRORG = {$organizationId}") > 0){
            $msgError .= Messages::ERR_EXISTS_EMAIL;
        }

        if($msgError){
            throw new \Exception($msgError);
        }

        $this->passwordValidate($password, $confirmPassword);
    }

    /**
     * @param $cdoperador
     * @param $dsemailoper
     * @throws Exception
     */
    public function operadorEditValidation($cdoperador, $dsemailoper)
    {
        $organizationId = getNrorg();
        $msgError = '';

        if($this->findCount("OPERADOR", "WHERE DSEMAILOPER = '{$dsemailoper}' AND NRORG = {$organizationId} AND CDOPERADOR != '{$cdoperador}'") > 0){
            $msgError .= Messages::ERR_EXISTS_EMAIL;
        }

        if($msgError){
            throw new \Exception($msgError);
        }
    }

    /**
     * @param $newPassword
     * @param $confirmPassword
     * @param null $password
     * @throws Exception
     */
    private function passwordValidate($newPassword, $confirmPassword, $password = null)
    {
        if($newPassword != $confirmPassword){
            throw new \Exception(Messages::ERR_PASSWORD_NOT_MATCH);
        }

        if($password && $password === $newPassword){
            throw new \Exception(Messages::ERR_NEW_PASSWORD_EQUAL_OLD);
        }
    }

    /**
     * @param $cpf
     * @return array|array[]
     * @throws \Doctrine\DBAL\Exception
     */
    private function operatorSaveValidationCpf($cpf): array
    {
        if ($cpf){
            $parcnegocioRepository = InstanceProvider::getRepository()->getParcnegocio();
            $cpfIsUsed = false;

            $parceegocios = $parcnegocioRepository->findBy(array('NRINSCRICAOPARC' => $cpf), getNrorg());

            foreach ($parceegocios as $pn) {
                $operators = $this->operadorRepository->findBy(array('NRPARCNEGOCIO' => $pn['NRPARCNEGOCIO']), getNrorg());
                if(!empty($operators)) {
                    $cpfIsUsed = true;
                    break;
                }
            }
            if($cpfIsUsed){
                throw new \Exception(Messages::ERR_EXISTS_CPF);
            }
            return $parceegocios;
        }
        return [];
    }

    /**
     * @param $nrinscricaoparc
     * @param $nrparcnegocio
     * @return array
     * @throws \Doctrine\DBAL\Exception
     */
    public function operatorEditValidationCpf($nrinscricaoparc, $nrparcnegocio): array
    {
        $organizationId = getNrorg();
        if($nrinscricaoparc){
            $cpfIsUsed = false;
            $parcnegocios = $this->select('PARCNEGOCIO', '*', "WHERE NRINSCRICAOPARC = '{$nrinscricaoparc}' AND NRORG = {$organizationId} AND NRPARCNEGOCIO != '{$nrparcnegocio}'");

            foreach ($parcnegocios as $pn) {
                $operadores = $this->operadorRepository->findBy(array('NRPARCNEGOCIO' => $pn['NRPARCNEGOCIO']), $organizationId);
                if(!empty($operadores)) {
                    $cpfIsUsed = true;
                    break;
                }
            }

            if($cpfIsUsed){
                throw new \Exception(Messages::ERR_EXISTS_CPF);
            }
            return $parcnegocios;
        }
        return [];
    }

    private function createOrganizacaooper($nrorg, $cdoperador, $cdoperadorInclusao)
    {
        $organizacaooperRepository = InstanceProvider::getRepository()->getOrganizacaooper();
        $nrorganizacaooper = InstanceProvider::getNewCode()->getCodeByTableName('ORGANIZACAOOPER', $nrorg);
        $idativo = 'S';
        $organizacaooperRepository->createOrganizacaooper($nrorganizacaooper, $nrorg, $cdoperador, $nrorg, getCurrentUserId(), $idativo);

    }

    /**
     * @param $nrparcnegocio
     * @param $nmoperador
     * @param $nrinscricaoparc
     * @param $cdtipoinscricao
     * @param $idativo
     * @param $parcnegocios
     * @param $entityManager
     * @return mixed
     * @throws \Doctrine\DBAL\Exception
     */
    private function setParcnegocio($nrparcnegocio, $nmoperador, $nrinscricaoparc, $cdtipoinscricao, $idativo, $parcnegocios, $entityManager, $labels)
    {
        if ($nrparcnegocio) {
            $operadores = $this->operadorRepository->findBy(array('NRPARCNEGOCIO' => $nrparcnegocio), getNrorg());

            if($operadores) {
                throw new \Exception(Messages::ERR_INVALID_PARCNEGOCIO);
            }
            return $nrparcnegocio;
        } else {
            if(empty($parcnegocios)) {
                $nrparcnegocio = InstanceProvider::getNewCode()->getCodeByTableName('PARCNEGOCIO', getNrorg());
                InstanceProvider::getService()->getParcnegocio()->createParcnegocio($nrparcnegocio, getNrorg(), $nmoperador, $nrinscricaoparc,
                    $cdtipoinscricao, getCurrentUserId(), $idativo);
                $valores = [$labels['label'], $nrparcnegocio];
                $nomes = ["Tela", $labels['NRPARCNEGOCIO']];
                $descricao =  AclUtilities::gerarDescricao($valores, $nomes);
                InstanceProvider::getLibLog()->logRegister($nrparcnegocio, $descricao, '1', 'PARCNEGOCIO', '/operator');
                $entityManager->flush();
                return $nrparcnegocio;
            } else {
                return $parcnegocios[0]['NRPARCNEGOCIO'];
            }
        }
    }

    /**
     * @param $nrparcnegocio
     * @param $nmoperador
     * @param $nrorg
     * @param $nrinscricaoparc
     * @param $cdtipoinscricao
     * @param $idativo
     * @param $parcnegocios
     * @param $oldCpf
     * @param $oldNmoperador
     * @param $entityManager
     * @return int|mixed|string
     * @throws Exception
     */
    private function setParcnegocioByEdit($nrparcnegocio, $nmoperador, $nrorg, $nrinscricaoparc, $cdtipoinscricao,
                                          $idativo, $parcnegocios, $oldCpf, $oldNmoperador, $entityManager, $labels)
    {

        if(!$nrparcnegocio) {
            if(empty($parcnegocios)) {
                $parcnegocioService = InstanceProvider::getService()->getParcnegocio();
                $nrparcnegocio = InstanceProvider::getNewCode()->getCodeByTableName('PARCNEGOCIO', $nrorg);
                $parcnegocioService->createParcnegocio($nrparcnegocio, $nrorg, $nmoperador, $nrinscricaoparc, $cdtipoinscricao, getCurrentUserId(), $idativo);
                $valores = [$labels['label'], $nrparcnegocio];
                $nomes = ["Tela", $labels['NRPARCNEGOCIO']];
                $descricao =  AclUtilities::gerarDescricao($valores, $nomes);
                InstanceProvider::getLibLog()->logRegister($nrparcnegocio, $descricao, '1', 'PARCNEGOCIO', '/operator');
                $entityManager->flush();
            } else {
                $nrparcnegocio = $nrparcnegocio[0]['NRPARCNEGOCIO'];
            }
            $parcnegocioService->createParceria($nrparcnegocio, $nrorg, getCurrentUserId(), 'OPERADOR', $labels);
            $entityManager->flush();
            return $nrparcnegocio;
        } else if ($nrinscricaoparc != $oldCpf || $nmoperador != $oldNmoperador){
            $parcnegocioRepository = InstanceProvider::getRepository()->getParcnegocio();
            $parcnegocioRepository->updateParcnegocio($nrinscricaoparc, $cdtipoinscricao, $nmoperador, $nrorg, getCurrentUserId(), $nrparcnegocio);
            $valores = [$labels['label'], $nrinscricaoparc];
            $nomes = ["Tela", $labels['NRINSCRICAOPARC']];
            $descricao =  AclUtilities::gerarDescricao($valores, $nomes);
            InstanceProvider::getLibLog()->logRegister($nrparcnegocio, $descricao, '2', 'PARCNEGOCIO', '/operator');
            $entityManager->flush();
        }
        return $nrparcnegocio;
    }

    /**
     * @return string
     * @throws \Doctrine\DBAL\Exception
     */
    private function generateCdoperador(): string
    {
        $newCode = InstanceProvider::getNewCode()->getCodeByTableName('OPERADOR'.getNrorg(), getNrorg(), 1, 10, true);

        if(strlen(getNrorg() . $newCode) >= 12){
            $newCode = substr($newCode, strlen(getNrorg()));
        }
        $cdoperador = getNrorg() . $newCode;
        $cdoperador = str_pad($cdoperador, 12, 0, STR_PAD_LEFT);
        $operadorEntity = InstanceProvider::getRepository()->getOperador()->findBy(array('CDOPERADOR' => $cdoperador));
        do{
            $cdoperador +=   1;
            $cdoperador = str_pad($cdoperador, 12, 0, STR_PAD_LEFT);
            $operadorEntity = InstanceProvider::getRepository()->getOperador()->findBy(array('CDOPERADOR' => $cdoperador));
        }
        while(!empty($operadorEntity));
        return $cdoperador;
    }

    private function setChangeClient($cdcliente, $nrparcnegocio, $entityManager)
    {
        $relacionaparcRepository = InstanceProvider::getRepository()->getRelacionaparc();
        $inteclieRepository = InstanceProvider::getRepository()->getInteclie();
        $parceriaRepository = InstanceProvider::getRepository()->getParceria();

        if($cdcliente == null) {
            $relacionaparcRepository->updateIdativoInRelacionaparc(getCurrentUserId(), getNrorg(), $nrparcnegocio, self::NOT_ACTIVE_ID);
            $entityManager->flush();
            $inteclieRepository->updateIdativoInteclie(getCurrentUserId(), getNrorg(), $nrparcnegocio);
            $entityManager->flush();
            $parceriaRepository->updateIdativoInParceria(getCurrentUserId(), getNrorg(), $nrparcnegocio, self::NOT_ACTIVE_ID);
            $entityManager->flush();
        } else {
            if($parceriaRepository->hasParceriaTypeInterlocutor(getNrorg(), $nrparcnegocio) == 0){
                $nrparceria = InstanceProvider::getNewCode()->getCodeByTableName('PARCERIA', getNrorg());
                $parceriaRepository->createParceria($nrparcnegocio, getNrorg(), getCurrentUserId(), 'INTERLOCUTOR', $nrparceria);
                $entityManager->flush();
            } else {
                $parceriaRepository->updateIdativoInParceria(getCurrentUserId(), getNrorg(), $nrparcnegocio, self::ACTIVE_ID);
                $entityManager->flush();
            }

            $relacionaparcRepository->updateIdativoInRelacionaparc(getCurrentUserId(), getNrorg(), $nrparcnegocio, self::NOT_ACTIVE_ID);
            $entityManager->flush();

            if($relacionaparcRepository->hasRelacionaparc(getNrorg(), $nrparcnegocio, $cdcliente) > 0){
                $inteclieRepository->activateInteclie(getNrorg(), $nrparcnegocio, $cdcliente, getCurrentUserId(), self::ACTIVE_ID);
                $entityManager->flush();
            }
        }
    }

    /**
     * @param $password
     * @param $passwordConfirm
     * @param $cdoperador
     * @param $email
     * @throws ConnectionException
     * @throws \Doctrine\DBAL\Driver\Exception
     */
    public function resetPassword($password, $passwordConfirm, $cdoperador, $email, $labels)
    {
        $connection = InstanceProvider::getEntityManager()->getConnection();
        $connection->beginTransaction();
        try {
            $this->passwordValidate($password, $passwordConfirm);
            $nrorg = getNrorg();
            $operador = $this->findOne('OPERADOR', '*', "WHERE CDOPERADOR = '$cdoperador' AND  NRORG = {$nrorg}");
            $passwordWeb    = md5($password);
            $passwordDelphi = substr(LibUtilities::getDelphiPasswordEncrypted($password), 0, 20);
            $oldPassword = $this->operadorRepository->findValAntigoOperators($cdoperador);
            $arrayValores = [$labels['label']];
            $arrayNomes = ['Tela'];
            if ($oldPassword[0]['CDSENHAOPERWEB'] !== $passwordWeb) {
                array_push($arrayValores, '*************');
                array_push($arrayNomes, $labels['NEW_PASSWORD']);
            }
            if ($oldPassword[0]['CDSENHOPER'] !== $passwordDelphi) {
                array_push($arrayValores, '*************');
                array_push($arrayNomes, 'CDSENHOPER');
            }
            if ($oldPassword[0]['IDSTATUSENHA'] !== 'A') {
                array_push($arrayValores, 'A');
                array_push($arrayNomes, 'IDSTATUSENHA');
            }
            $descricao = AclUtilities::gerarDescricao($arrayValores, $arrayNomes);
            InstanceProvider::getLibLog()->logRegister($cdoperador, $descricao, '2', 'OPERADOR', '/resetPassword/save');
            $this->operadorRepository->resetPassword($passwordWeb, $passwordDelphi, $cdoperador, $nrorg);
            InstanceProvider::getRepository()->getLibUserData()->insertControlSenha($operador['CDOPERADOR'], $nrorg, $passwordDelphi, $passwordWeb, 'R');
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    /**
     * @return array
     */
    public function findOperadorInfo(): array
    {
        $operadorInfo = array(
            'CDOPERADOR' => ACLSession::getCdoperador(),
            'NMOPERADOR' => ACLSession::getNmoperador(),
            'NRORG' => ACLSession::getNrorg(),
            'NMORG' => ACLSession::getNmorg(),
            'CDFILIAL' => ['0002', '9999'],
            'NRORGTRAB' => ACLSession::getNrorgtrab(),
            'NMORGTRAB' => null,
            'NRESTRUTURAM' => ACLSession::getNrestruttrab(),
            'NMESTRUTURAM' => null,
            'ACL_ADMINISTRATOR' => ACLSession::getACLAdministrator(),
            'OTHER_INFORMATION' => array(
                'IDUTCTRLACESWEB' => ACLSession::getIdutctrlacesweb(),
                'IDUTLPERFILFILIAL' => ACLSession::getIdutlperfilfilial(),
            )
        );

        $organizacao = $this->findOne('ORGANIZACAO', 'NMORGANIZACAO', "WHERE NRORG = {$operadorInfo['NRORGTRAB']}");
        $idopermaster = $this->findOne('OPERADOR', 'IDOPERMASTER', "WHERE CDOPERADOR = '{$operadorInfo['CDOPERADOR']}'");
        
        $operadorInfo['NMORGTRAB'] = $organizacao['NMORGANIZACAO'];
        $operadorInfo['IDOPERMASTER'] = $idopermaster['IDOPERMASTER'];

        if(!empty($operadorInfo['NRESTRUTURAM'])) {
            $estruturam = $this->findOne('ESTRUTURAM', 'NMESTRUTURAM', "WHERE NRESTRUTURAM = {$operadorInfo['NRESTRUTURAM']}");
            if(!empty($estruturam)) {
                $operadorInfo['NMESTRUTURAM'] = $estruturam['NMESTRUTURAM'];
            }
        }

        return $operadorInfo;
    }

    public function findLogOpera(): array
    {
        return $this->operadorRepository->findLogOpera(getNrorg());
    }
}
